/*   
 *   $Id: calc.c,v 1.5 2005/06/30 18:54:37 ondra Exp $
 * 
 *   Holiday Debts manager for PalmOS.
 *   Copyright (C) 2000,2001,2005 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *   You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>

#include "debtsRsc.h"
#include "calc.h"
#include "float.h"


static double numstack[3];
static int numposition;

int Operation,DefOperation;
/* Number of decimal places to show */
int Precision;
/* If the user presses operator, use the displayed number,
 * otherwise let the user write a new number 
 */
Boolean resetnum;
/* Set if the operation button was pressed. Evade nasty things
 * when operation button is pressed twice */
Boolean operInProgress;
int dotmult, dotpos;

#define MAXNUM 1E9

/*
 * Display a number from numstack[indx] to result area.
 */
static void SetNumber(FormPtr frm, int index) CALC_SECTION;
static void
SetNumber(FormPtr frm, int index)
{
    static char numstr[20];
    int i,j;
    FieldPtr field = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,fldCalcResult));
    
    if (numstack[index] < -MAXNUM || numstack[index]>MAXNUM) 
	StrCopy(numstr, "Error");
    else {
	float_to_str(numstr,numstack[index],Precision);
	/* Remove trailing zeros */
	for (i=StrLen(numstr)-1,j=0;numstr[i] == '0' && j<(Precision-dotpos+1);--i,++j)
	    numstr[i] = '\0';
	if (!dotpos && (numstr[i] == '.' || numstr[i] == ','))
	    numstr[i] = '\0';
    }
    FldSetTextPtr(field,numstr);
    if (FrmVisible(frm))
	FldDrawField(field);
}

/*
 * Do operation on numstack. Operation is defined by the
 * codes of the operation buttons
 */
static void do_operation(int i1, int i2, int operation) CALC_SECTION;
static void
do_operation(int i1, int i2, int operation)
{
    switch (operation) {
    case btnCalcPlus:
	numstack[i1] += numstack[i2];
	break;
    case btnCalcMinus:
	numstack[i1] -= numstack[i2];
	break;
    case btnCalcMult:
	numstack[i1] *= numstack[i2];
	break;
    case btnCalcDivide:
	numstack[i1] /= numstack[i2];
	break;
    }
}

/*
 * Event handler for calculator
 */
static Boolean CalcFormHandleEvent (EventPtr e) CALC_SECTION;
static Boolean 
CalcFormHandleEvent (EventPtr e)
{
    Boolean handled = false;
    
    switch (e->eType) {
    case ctlSelectEvent:
	switch (e->data.ctlSelect.controlID) {
	case btnCalc0:
	case btnCalc1:
	case btnCalc2:
	case btnCalc3:
	case btnCalc4:
	case btnCalc5:
	case btnCalc6:
	case btnCalc7:
	case btnCalc8:
	case btnCalc9:
	    handled = true;
	    operInProgress = false;
	    if (resetnum) {
		numstack[numposition] = 0.0;
		resetnum = false;
	    }
	    if (numstack[numposition] > MAXNUM || numstack[numposition] < -MAXNUM) 
		break;
	    if (dotpos) {
		if (dotpos <= Precision) {
		    numstack[numposition] += ((double)e->data.ctlSelect.controlID - btnCalc0) / dotmult;
		    dotmult *= 10;
		    dotpos++;
		}
	    } else {
		numstack[numposition] *= 10;
		numstack[numposition] += e->data.ctlSelect.controlID - btnCalc0;
	    }
	    SetNumber(FrmGetActiveForm(),numposition);
	    break;
	case btnCalcPlus:
	case btnCalcMinus:
	    if (operInProgress)
		break;
	    if (numposition == 0) {
		numposition = 1;
		numstack[numposition] = 0.0;
		Operation = e->data.ctlSelect.controlID;
	    } else if (numposition == 1) {
		do_operation(0,1,Operation);
		Operation = e->data.ctlSelect.controlID;
		SetNumber(FrmGetActiveForm(),0);
		numstack[1] = 0.0;
	    } else if (numposition == 2) {
		do_operation(1,2,DefOperation);
		do_operation(0,1,Operation);
		Operation = e->data.ctlSelect.controlID;
		SetNumber(FrmGetActiveForm(),0);
		numposition = 1;
		numstack[1] = 0.0;
	    }
	    dotpos = 0;
	    operInProgress = true;
	    handled = true;
	    break;
	case btnCalcMult:	    
	case btnCalcDivide:
	    if (operInProgress)
		break;
	    if (numposition == 0) {
		numposition = 1;
		numstack[numposition] = 0.0;
		Operation = e->data.ctlSelect.controlID;
	    } else if (numposition == 1 && 
		       (Operation==btnCalcMult || Operation ==btnCalcDivide)) {
		do_operation(0,1,Operation);
		Operation = e->data.ctlSelect.controlID;
		SetNumber(FrmGetActiveForm(),0);
		numstack[1] = 0.0;
	    } else if (numposition == 1) {
		DefOperation = e->data.ctlSelect.controlID;
		numstack[2] = 0.0;
		numposition = 2;
	    } else if (numposition == 2) {
		do_operation(1,2,DefOperation);
		SetNumber(FrmGetActiveForm(),1);
		numstack[2] = 0.0;
	    }
	    dotpos = 0;
	    operInProgress = true;
	    handled = true;
	    break;
	case btnCalcExe:
	    handled = true;

	    if (operInProgress)
		break;
	    switch (numposition) {
	    case 0:
		break;
	    case 1:
		do_operation(0,1,Operation);
		SetNumber(FrmGetActiveForm(), 0);
		break;
	    case 2:
		do_operation(1,2,DefOperation);
		do_operation(0,1,Operation);
		SetNumber(FrmGetActiveForm(), 0);		
		break;		
	    }
	    dotpos = 0;
	    numposition = 0;
	    resetnum = true;
	    break;
	case btnCalcDot:
	    if (resetnum) {
		numstack[numposition] = 0.0;
		resetnum = false;
	    }
	    if (!dotpos) {
		dotpos = 1;
		dotmult = 10;
	    }
	    SetNumber(FrmGetActiveForm(), numposition);
	    handled = true;
	    break;
	case btnCalcC:
	    operInProgress = false;
	    resetnum = false;
	    numstack[0] = 0.0;
	    numposition = 0;
	    dotpos = 0;
	    SetNumber(FrmGetActiveForm(), 0);
	    handled = true;
	    break;
	}	
    default:
	break;
    }
    
    return handled;
}    

/*
 * Show a popup calculator, initialize it to value that is
 * written in the @field, allow inserting result of calculation
 * back to the field
 */
Boolean
calc_compute(FieldPtr field, int precision)
{
    FormPtr frm;
    int prefcomma = PrefGetPreference(prefNumberFormat);
    char *fldtxt;

    Precision = precision;

    frm = FrmInitForm(frmCalculator);
    FrmSetEventHandler(frm, CalcFormHandleEvent);

    dotpos = 0;
    operInProgress = false;
    resetnum = false;
    numposition = 0;    
    numstack[0] = 0.0;
    
    /* Read value from original text */
    fldtxt = FldGetTextPtr(field);
    if (fldtxt) {
	numstack[0] = float_to_double(fldtxt);
	resetnum = true;
    }
    
    /* Set dot or comma according to locale on the 'dot' button */
    switch (prefcomma) {
    case nfPeriodComma:
    case nfSpaceComma:
    case nfApostropheComma:
	CtlSetLabel(FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,btnCalcDot)), ",");
	break;
    default:
	break;
    }
    
    /* Display number to the result area */
    SetNumber(frm, 0);

    /* Show form */
    if (FrmDoDialog(frm) == btnCalcInsert) {
	char *txt;
	txt = FldGetTextPtr(FrmGetObjectPtr(frm,FrmGetObjectIndex(frm, fldCalcResult)));
	if (txt) {
	    char *tmp2;
	    tmp2 = FldGetTextPtr(field);
	    if (tmp2) 
		FldDelete(field,0,StrLen(tmp2));
	    FldInsert(field, txt, StrLen(txt));
	}
    }

    FrmDeleteForm(frm);
    
    return true;
}
