/*   
 *   $Id: dbmgr.c,v 1.18 2005/03/24 18:34:05 ondra Exp $
 * 
 *   Holiday Debts manager for PalmOS.
 *   Copyright (C) 2000,2001 Ondrej Palkovsky
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * 
 *   You can contact me at 'ondrap@penguin.cz'.
*/

#include <PalmOS.h>
#include "debts.h"
#include "debtsRsc.h"
#include "main.h"
#include "txtask.h"

static struct {
    Int16 nitems;
    struct {
	char name[dmDBNameLength+1];
	UInt16 cardno;
	LocalID dbid;
    }*db;
}dbList = {0,NULL};

char **nList = NULL;
static Int16 selectedLine = -1;
static LocalID lastDbID = 0;

void
dbmgr_set_last_db(LocalID lastid)
{
    lastDbID = lastid;
}

static void
dbmgr_destroy(void)
{
    if (dbList.db != NULL) {
	MemPtrFree(dbList.db);
	dbList.db = NULL;
    }
    if (nList != NULL) {
	MemPtrFree(nList);
	nList = NULL;
    }
}

static void
dbmgr_update(void)
{
    DmSearchStateType state;
    UInt16 cardno;
    LocalID dbid;
    Err err;
    Int16 dbcount;
    Int16 gap,count,i,j;

    dbmgr_destroy();

    err = DmGetNextDatabaseByTypeCreator(true,&state,DBTYPE,APP_ID,false,
					 &cardno,&dbid);
    dbcount = 0;
    while (!err && dbid) {
	dbcount++;
	err = DmGetNextDatabaseByTypeCreator(false,&state,DBTYPE,APP_ID,false,
					     &cardno,&dbid);
    }
    /* Avoid zero allocation, allow last item to be used as swap
       for sorting */
    dbList.db = MemPtrNew((dbcount+1)*sizeof(*dbList.db));
    nList = MemPtrNew(dbcount*sizeof(nList[0]) + 1);

    err = DmGetNextDatabaseByTypeCreator(true,&state,DBTYPE,APP_ID,false,
					 &cardno,&dbid);
    dbList.nitems = 0;
    while (!err && dbid) {
	dbList.db[dbList.nitems].cardno = cardno;
	dbList.db[dbList.nitems].dbid = dbid;
	DmDatabaseInfo(cardno,dbid,dbList.db[dbList.nitems].name,
		       NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
	dbList.nitems++;
	err = DmGetNextDatabaseByTypeCreator(false,&state,DBTYPE,APP_ID,false,
					     &cardno,&dbid);
    }

    /* Shell sort it */
    count = dbList.nitems;
    for (gap = count/2;gap  > 0;gap /= 2)
	for (i = gap;i < count;i++)
	    for (j=i-gap;
		 j>=0 && StrCaselessCompare(dbList.db[j].name,
					    dbList.db[j+gap].name)>0;
		 j-=gap) {
		/* Swap it using last item as temp */
		dbList.db[count] = dbList.db[j];
		dbList.db[j] = dbList.db[j+gap];
		dbList.db[j+gap] = dbList.db[count];
	    }
    /* Set up the name table */
    for (i=0;i<dbcount;i++)
	nList[i] = dbList.db[i].name+4; /* Do not display the 'OpDt' prefix */
    LstSetListChoices(GetObjectPtr(lstDBmgr),nList,dbList.nitems);
    /* Select the last used database */
    for (i=0;i<dbcount;i++)
	if (dbList.db[i].dbid == lastDbID) {
	    LstSetSelection(GetObjectPtr(lstDBmgr),i);
	    lastDbID = 0;
	    break;
	}
}

static void
dbmgr_rename_db(LocalID dbid,Int16 cardno)
{
    char dbname[dmDBNameLength+1];
    Err err;
    Boolean ok;

    /* We always prefix the database with our application ID */
    StrCopy(dbname,"OpDt");
    ok = txtask_dialog(strNewDb,strDbName,dbname+4,dmDBNameLength-4);
    if (ok) {
	err = DmSetDatabaseInfo(cardno,dbid,
				dbname,NULL,NULL,
				NULL,NULL,
				NULL,NULL,
				NULL,NULL,
				NULL,NULL);
	if (err)
	    FrmAlert(altDBRenameFailed);
	else
	    FrmUpdateForm(frmDBmgr,updDBmgrDBlist);
    }
}    
    
static Boolean
dbmgr_new_db(LocalID *dbid,Int16 *cardno)
{
    char dbname[dmDBNameLength+1];
    Boolean ok;
    
    /* We always prefix the database with our application ID */
    StrCopy(dbname,"OpDt");
    ok = txtask_dialog(strNewDb,strDbName,dbname+4,dmDBNameLength-4);
    if (ok) {
	ok = db_create_new(dbname,dbid,cardno);
	if (!ok)
	    FrmAlert(altDBCreateFailed);
    }

    return ok;
}    

static Err
dbmgr_beam_write(const void* dataP,UInt32 *sizeP,void *userDataP)
{
    Err err;

    *sizeP = ExgSend( (ExgSocketPtr) userDataP, dataP, *sizeP, &err ); 
    return err;
}

/***********************************************************************
 *
 * FUNCTION:     dbmbr_beam_db
 * 
 * DESCRIPTION:  Beam selected database to other device
 *
 * PARAMETERS:   dbid,cardno
 *
 * RETURN:       
 *      
 ***********************************************************************/
static void
dbmgr_beam_db(LocalID dbid,UInt16 cardno)
{
    ExgSocketType socket;
    char dbname[dmDBNameLength+1];
    char filename[dmDBNameLength+1+4];
    Err err;

    if (DmDatabaseInfo(cardno, dbid, dbname,NULL,NULL,NULL,
		       NULL,NULL,NULL,NULL,
		       NULL,NULL,NULL))
	return;

    StrCopy(filename,dbname);
    StrCat(filename,".pdb");

    MemSet(&socket,sizeof(socket),0);
    socket.name = filename;
    socket.type="application/vnd.palm";
    socket.description=dbname+4;

    err = ExgPut(&socket);
    if (err != errNone) 
	return;
    ExgDBWrite(dbmgr_beam_write, &socket, filename, dbid, cardno);
    ExgDisconnect(&socket, err);
}

/***********************************************************************
 *
 * FUNCTION:     dbmgr_resize_form
 * 
 * DESCRIPTION:  Resize form to match large screen
 *
 * PARAMETERS:   Form, fromBounds, toBounds
 *
 * RETURN:       Void
 *      
 ***********************************************************************/
static Boolean
dbmgr_resize_form(FormType *frm)
{
    RectangleType fromBounds, toBounds;
    Int16 heightdelta, widthdelta;
    Int16 idx,i;
    Coord x,y;
    RectangleType objbounds;
    Int16 bottomIds[] = {btnDBmgrCreate, btnDBmgrCreateAs,
			 btnDBmgrDelete, 0};
    ListPtr lst;

    if (!palmOS40)
	return false;

    WinGetBounds(FrmGetWindowHandle(frm), &fromBounds);
    // Get new display bounds
    WinGetBounds(WinGetDisplayWindow(), &toBounds);


    heightdelta = (toBounds.extent.y - toBounds.topLeft.y) - (fromBounds.extent.y - fromBounds.topLeft.y);
    widthdelta = (toBounds.extent.x - toBounds.topLeft.x) - (fromBounds.extent.x - fromBounds.topLeft.x);
    
    if (!heightdelta && !widthdelta)
	return false;
    WinSetBounds(FrmGetWindowHandle(frm), &toBounds);

    /* Go through particular objects and move them */
    for (i=0; bottomIds[i];i++) {
	idx = FrmGetObjectIndex(frm, bottomIds[i]);
	FrmGetObjectPosition(frm, idx, &x, &y);
	FrmSetObjectPosition(frm, idx, x, y+heightdelta);
    }
    /* Table */
    idx = FrmGetObjectIndex(frm, lstDBmgr);
    FrmGetObjectBounds(frm, idx, &objbounds);
    objbounds.extent.x += widthdelta;
    objbounds.extent.y += heightdelta;
    FrmSetObjectBounds(frm, idx, &objbounds);
    lst = FrmGetObjectPtr(frm, idx);
    /* Magic number 11px = 1 row, at least 10 (some rounding present) */
    i = objbounds.extent.y / 11;
    if (i < 10)
	i = 10;
    LstSetHeight(lst, i);

    return true;
}


/***********************************************************************
 *
 * FUNCTION:     DBmgrFormHandleEvent
 * 
 * DESCRIPTION:  Event handler for Person form
 *
 * PARAMETERS:   event
 *
 * RETURN:       handled
 *      
 ***********************************************************************/
Boolean
DBmgrFormHandleEvent(EventPtr e)
{
    Boolean handled = false;
    FormPtr frm;
    Int16 i;
    LocalID dbid;
    Int16 cardno;
   
    switch (e->eType) {
    case frmOpenEvent:
	selectedLine = -1;
	frm = FrmGetActiveForm();
	dbmgr_update();

	dbmgr_resize_form(frm);

	FrmDrawForm(frm);
	handled = true;
	break;
    case frmCloseEvent:
	dbmgr_destroy();
	handled = false;
	break;
    case frmSaveEvent:
	debts_save_dbmgr();
	handled = true;
	break;
    case menuEvent:
	MenuEraseStatus(NULL);
	switch(e->data.menu.itemID) {
	case menuHelp:
	    FrmHelp(strHelp);
	    handled = true;
	    break;
	case menuDBBeam:
	    i = LstGetSelection(GetObjectPtr(lstDBmgr));
	    if (i != noListSelection) 
		dbmgr_beam_db(dbList.db[i].dbid,dbList.db[i].cardno);
	    handled = true;
	    break;
	case menuDBRename:
	    i = LstGetSelection(GetObjectPtr(lstDBmgr));
	    if (i != noListSelection) 
		dbmgr_rename_db(dbList.db[i].dbid,dbList.db[i].cardno);
	    handled = true;
	    break;
	case menuDBAbout:
	    FrmAlert(altAbout);
	    handled = true;
	    break;
	case menuDBPrefs:
	    txtask_dialog_cbox(strPreferences,strDefaultCurrency,
			       strActiveBalance,
			       defaultCurrency,MAX_CURRENCY_NAME,
			       &activeBalance);
	    break;
	}
	break;
    case lstSelectEvent:
	i = e->data.lstSelect.selection;
	if (i == selectedLine) {
	    main_opendb(dbList.db[i].dbid,dbList.db[i].cardno);
	} else
	    selectedLine = i;
	handled = true;
	break;
    case ctlSelectEvent:
	switch(e->data.ctlSelect.controlID) {
	case btnDBmgrCreate:
	    if (dbmgr_new_db(&dbid,&cardno)) 
		main_opendb(dbid,cardno);
	    handled = true;
	    break;
	case btnDBmgrCreateAs:
	    i = LstGetSelection(GetObjectPtr(lstDBmgr));
	    if (i != noListSelection) 
		if (dbmgr_new_db(&dbid,&cardno)) {
		    db_copy_template(dbid,cardno,
				     dbList.db[i].dbid,dbList.db[i].cardno);
		    main_opendb(dbid,cardno);
		}
	    handled = true;
	    break;
	case btnDBmgrDelete:
	    i = LstGetSelection(GetObjectPtr(lstDBmgr));
	    if (i != noListSelection) 
		if (FrmCustomAlert(altDeleteDBOK,dbList.db[i].name+4,NULL,NULL) == 0) {
		    db_delete_database(dbList.db[i].dbid,dbList.db[i].cardno);
		    FrmUpdateForm(frmDBmgr,updDBmgrDBlist);
		}
	    handled = true;
	    break;
	}
	break;
    case frmUpdateEvent:
	if (e->data.frmUpdate.updateCode == updDBmgrDBlist) {
	    dbmgr_update();
	    LstDrawList(GetObjectPtr(lstDBmgr));
	    handled = true;
	}
	break;
    case winDisplayChangedEvent:
	if (dbmgr_resize_form(FrmGetActiveForm()))
	    FrmDrawForm(FrmGetActiveForm());
	handled = true;
	break;
    default:
        break;
    }

    return handled;
}

